declare abstract class AbstractObj {
    abstract toString(): string;
    toJSON(): string;
}
declare abstract class AbstractNoValueObj extends AbstractObj {
    valueOf(): void;
}
declare abstract class AbstractISOObj<ISOFields> extends AbstractNoValueObj {
    constructor(isoFields: ISOFields);
    getISOFields(): ISOFields;
}

declare class Duration extends AbstractNoValueObj {
    constructor(years?: number, months?: number, weeks?: number, days?: number, hours?: number, minutes?: number, seconds?: number, milliseconds?: number, microseconds?: number, nanoseconds?: number);
    static from(arg: DurationArg): Duration;
    static compare(a: DurationArg, b: DurationArg, options?: {
        relativeTo?: ZonedDateTimeArg | DateTimeArg;
    }): CompareResult;
    get years(): number;
    get months(): number;
    get weeks(): number;
    get days(): number;
    get hours(): number;
    get minutes(): number;
    get seconds(): number;
    get milliseconds(): number;
    get microseconds(): number;
    get nanoseconds(): number;
    get sign(): CompareResult;
    get blank(): boolean;
    with(fields: DurationLike): Duration;
    negated(): Duration;
    abs(): Duration;
    add(other: DurationArg, options?: {
        relativeTo?: ZonedDateTimeArg | DateTimeArg;
    }): Duration;
    subtract(other: DurationArg, options?: {
        relativeTo?: ZonedDateTimeArg | DateTimeArg;
    }): Duration;
    round(options: DurationRoundingOptions): Duration;
    total(options: DurationTotalOptions): number;
    toString(options?: DurationToStringOptions): string;
    toLocaleString(_locales?: LocalesArg, _options?: unknown): string;
}

interface OverflowHandlingMap {
    constrain: 0;
    reject: 1;
}

declare class Instant extends AbstractNoValueObj {
    constructor(epochNanoseconds: bigint);
    static from(arg: InstantArg): Instant;
    static fromEpochSeconds(epochSeconds: number): Instant;
    static fromEpochMilliseconds(epochMilliseconds: number): Instant;
    static fromEpochMicroseconds(epochMicroseconds: bigint): Instant;
    static fromEpochNanoseconds(epochNanoseconds: bigint): Instant;
    static compare(a: InstantArg, b: InstantArg): CompareResult;
    get epochNanoseconds(): bigint;
    add(durationArg: DurationArg): Instant;
    subtract(durationArg: DurationArg): Instant;
    until(other: InstantArg, options?: TimeDiffOptions): Duration;
    since(other: InstantArg, options?: TimeDiffOptions): Duration;
    round(options: TimeRoundingOptions): Instant;
    equals(other: InstantArg): boolean;
    toString(options?: InstantToStringOptions): string;
    toLocaleString(locales?: LocalesArg, options?: Intl.DateTimeFormatOptions): string;
    toZonedDateTimeISO(timeZoneArg: TimeZoneArg): ZonedDateTime;
    toZonedDateTime(options: {
        calendar: CalendarArg;
        timeZone: TimeZoneArg;
    }): ZonedDateTime;
}
interface Instant extends ComputedEpochFields {
}

declare class PlainMonthDay extends AbstractISOObj<DateISOFields> {
    constructor(isoMonth: number, isoDay: number, calendar?: CalendarProtocol, referenceISOYear?: number);
    static from(arg: MonthDayArg, options?: OverflowOptions): PlainMonthDay;
    with(fields: MonthDayOverrides, options?: OverflowOptions): PlainMonthDay;
    equals(other: MonthDayArg): boolean;
    toString(options?: DateToStringOptions): string;
    toLocaleString(locales?: LocalesArg, options?: Intl.DateTimeFormatOptions): string;
    toPlainDate(fields: {
        year: number;
    }, options?: OverflowOptions): PlainDate;
}
interface PlainMonthDay extends MonthDayCalendarFields {
    calendar: Calendar;
}

declare class PlainYearMonth extends AbstractISOObj<DateISOFields> {
    constructor(isoYear: number, isoMonth: number, calendarArg?: CalendarArg, referenceISODay?: number);
    static from(arg: YearMonthArg, options?: OverflowOptions): PlainYearMonth;
    static compare(a: YearMonthArg, b: YearMonthArg): CompareResult;
    with(fields: YearMonthOverrides, options?: OverflowOptions): PlainYearMonth;
    add(durationArg: DurationArg, options?: OverflowOptions): PlainYearMonth;
    subtract(durationArg: DurationArg, options?: OverflowOptions): PlainYearMonth;
    until(other: YearMonthArg, options?: YearMonthDiffOptions): Duration;
    since(other: YearMonthArg, options?: YearMonthDiffOptions): Duration;
    equals(other: YearMonthArg): boolean;
    toString(options?: DateToStringOptions): string;
    toLocaleString(locales?: LocalesArg, options?: Intl.DateTimeFormatOptions): string;
    toPlainDate(fields: {
        day: number;
    }): PlainDate;
}
interface PlainYearMonth extends YearMonthCalendarFields {
    calendar: Calendar;
}

declare class TimeZone extends AbstractObj implements TimeZoneProtocol {
    constructor(id: string);
    static from(arg: TimeZoneArg): TimeZone;
    get id(): string;
    getOffsetStringFor(instantArg: InstantArg): string;
    getOffsetNanosecondsFor(instantArg: InstantArg): number;
    getPlainDateTimeFor(instantArg: InstantArg, calendarArg?: CalendarArg): PlainDateTime;
    getInstantFor(dateTimeArg: DateTimeArg, options?: {
        disambiguation?: Disambiguation;
    }): Instant;
    getPossibleInstantsFor(dateTimeArg: DateTimeArg): Instant[];
    getPreviousTransition(instantArg: InstantArg): Instant | null;
    getNextTransition(instantArg: InstantArg): Instant | null;
    toString(): string;
}

declare class ZonedDateTime extends AbstractISOObj<ZonedDateTimeISOFields> {
    constructor(epochNanoseconds: bigint, timeZoneArg: TimeZoneArg, calendarArg?: CalendarArg);
    static from(arg: ZonedDateTimeArg, options?: ZonedDateTimeOptions): ZonedDateTime;
    static compare(a: ZonedDateTimeArg, b: ZonedDateTimeArg): CompareResult;
    get timeZone(): TimeZone;
    get epochNanoseconds(): bigint;
    get offsetNanoseconds(): number;
    get offset(): string;
    with(fields: ZonedDateTimeOverrides, options?: ZonedDateTimeOptions): ZonedDateTime;
    withPlainDate(dateArg: DateArg): ZonedDateTime;
    withPlainTime(timeArg?: TimeArg): ZonedDateTime;
    withCalendar(calendarArg: CalendarArg): ZonedDateTime;
    withTimeZone(timeZoneArg: TimeZoneArg): ZonedDateTime;
    add(durationArg: DurationArg, options?: OverflowOptions): ZonedDateTime;
    subtract(durationArg: DurationArg, options?: OverflowOptions): ZonedDateTime;
    until(other: ZonedDateTimeArg, options?: DiffOptions): Duration;
    since(other: ZonedDateTimeArg, options?: DiffOptions): Duration;
    round(options?: DateTimeRoundingOptions): ZonedDateTime;
    equals(other: ZonedDateTimeArg): boolean;
    startOfDay(): ZonedDateTime;
    get hoursInDay(): number;
    toString(options?: ZonedDateTimeToStringOptions): string;
    toLocaleString(locales?: LocalesArg, options?: Intl.DateTimeFormatOptions): string;
    toPlainYearMonth(): PlainYearMonth;
    toPlainMonthDay(): PlainMonthDay;
    toPlainDateTime(): PlainDateTime;
    toPlainDate(): PlainDate;
    toPlainTime(): PlainTime;
    toInstant(): Instant;
}
interface ZonedDateTime extends DateCalendarFields {
    calendar: Calendar;
}
interface ZonedDateTime extends TimeFields {
}
interface ZonedDateTime extends ComputedEpochFields {
}

declare class PlainTime extends AbstractISOObj<TimeISOFields> {
    constructor(isoHour?: number, isoMinute?: number, isoSecond?: number, isoMillisecond?: number, isoMicrosecond?: number, isoNanosecond?: number);
    static from(arg: TimeArg, options?: OverflowOptions): PlainTime;
    static compare(a: TimeArg, b: TimeArg): CompareResult;
    with(fields: TimeLike, options?: OverflowOptions): PlainTime;
    add(durationArg: DurationArg): PlainTime;
    subtract(durationArg: DurationArg): PlainTime;
    until(other: TimeArg, options?: TimeDiffOptions): Duration;
    since(other: TimeArg, options?: TimeDiffOptions): Duration;
    round(options: TimeRoundingOptions): PlainTime;
    equals(other: TimeArg): boolean;
    toString(options?: TimeToStringOptions): string;
    toLocaleString(locales?: LocalesArg, options?: Intl.DateTimeFormatOptions): string;
    toZonedDateTime(options: {
        plainDate: DateArg;
        timeZone: TimeZoneArg;
    }): ZonedDateTime;
    toPlainDateTime(dateArg: DateArg): PlainDateTime;
}
interface PlainTime extends TimeFields {
    calendar: Calendar;
}

interface TimeISOMilli {
    isoHour: number;
    isoMinute: number;
    isoSecond: number;
    isoMillisecond: number;
}
interface TimeISOEssentials extends TimeISOMilli {
    isoMicrosecond: number;
    isoNanosecond: number;
}
interface TimeFields {
    hour: number;
    minute: number;
    second: number;
    millisecond: number;
    microsecond: number;
    nanosecond: number;
}

declare class PlainDateTime extends AbstractISOObj<DateTimeISOFields> {
    constructor(isoYear: number, isoMonth: number, isoDay: number, isoHour?: number, isoMinute?: number, isoSecond?: number, isoMillisecond?: number, isoMicrosecond?: number, isoNanosecond?: number, calendarArg?: CalendarArg);
    static from(arg: DateTimeArg, options?: OverflowOptions): PlainDateTime;
    static compare(a: DateTimeArg, b: DateTimeArg): CompareResult;
    with(fields: DateTimeOverrides, options?: OverflowOptions): PlainDateTime;
    withPlainDate(dateArg: DateArg): PlainDateTime;
    withPlainTime(timeArg?: TimeArg): PlainDateTime;
    withCalendar(calendarArg: CalendarArg): PlainDateTime;
    add(durationArg: DurationArg, options?: OverflowOptions): PlainDateTime;
    subtract(durationArg: DurationArg, options?: OverflowOptions): PlainDateTime;
    until(other: DateTimeArg, options?: DiffOptions): Duration;
    since(other: DateTimeArg, options?: DiffOptions): Duration;
    round(options: DateTimeRoundingOptions): PlainDateTime;
    equals(other: DateTimeArg): boolean;
    toString(options?: DateTimeToStringOptions): string;
    toLocaleString(locales?: LocalesArg, options?: Intl.DateTimeFormatOptions): string;
    toZonedDateTime(timeZoneArg: TimeZoneArg, options?: {
        disambiguation?: Disambiguation;
    }): ZonedDateTime;
    toPlainYearMonth(): PlainYearMonth;
    toPlainMonthDay(): PlainMonthDay;
    toPlainDate(): PlainDate;
    toPlainTime(): PlainTime;
}
interface PlainDateTime extends DateCalendarFields {
    calendar: Calendar;
}
interface PlainDateTime extends TimeFields {
}

declare type RoundingFunc = (n: number) => number;

interface YearMonthFields {
    era: string | undefined;
    eraYear: number | undefined;
    year: number;
    month: number;
    monthCode: string;
}

declare type DateISOEssentials = {
    isoYear: number;
    isoMonth: number;
    isoDay: number;
};
declare type DateFields = YearMonthFields & {
    day: number;
};

interface ComputedEpochFields {
    epochMicroseconds: bigint;
    epochMilliseconds: number;
    epochSeconds: number;
}
interface YearMonthCalendarFields {
    era: string | undefined;
    eraYear: number | undefined;
    year: number;
    month: number;
    monthCode: string;
    daysInWeek: number;
    daysInMonth: number;
    daysInYear: number;
    monthsInYear: number;
    inLeapYear: boolean;
}
interface MonthDayCalendarFields {
    monthCode: string;
    day: number;
}
interface DateCalendarFields extends YearMonthCalendarFields {
    day: number;
    dayOfWeek: number;
    dayOfYear: number;
    weekOfYear: number;
}

declare class PlainDate extends AbstractISOObj<DateISOFields> {
    constructor(isoYear: number, isoMonth: number, isoDay: number, calendarArg?: CalendarArg);
    static from(arg: DateArg, options?: OverflowOptions): PlainDate;
    static compare(a: DateArg, b: DateArg): CompareResult;
    with(fields: DateOverrides, options?: OverflowOptions): PlainDate;
    withCalendar(calendarArg: CalendarArg): PlainDate;
    add(durationArg: DurationArg, options?: OverflowOptions): PlainDate;
    subtract(durationArg: DurationArg, options?: OverflowOptions): PlainDate;
    until(other: DateArg, options?: DateDiffOptions): Duration;
    since(other: DateArg, options?: DateDiffOptions): Duration;
    equals(other: DateArg): boolean;
    toString(options?: DateToStringOptions): string;
    toLocaleString(locales?: LocalesArg, options?: Intl.DateTimeFormatOptions): string;
    toZonedDateTime(options: {
        plainTime?: TimeArg;
        timeZone: TimeZoneArg;
    }): ZonedDateTime;
    toPlainDateTime(timeArg?: TimeArg): PlainDateTime;
    toPlainYearMonth(): PlainYearMonth;
    toPlainMonthDay(): PlainMonthDay;
}
interface PlainDate extends DateCalendarFields {
    calendar: Calendar;
}

declare class Calendar extends AbstractObj implements CalendarProtocol {
    constructor(id: string);
    static from(arg: CalendarArg): Calendar;
    get id(): string;
    era(arg: PlainYearMonth | DateArg | PlainDateTime | ZonedDateTime): string | undefined;
    eraYear(arg: PlainYearMonth | DateArg | PlainDateTime | ZonedDateTime): number | undefined;
    year(arg: PlainYearMonth | DateArg | PlainDateTime | ZonedDateTime): number;
    month(arg: PlainYearMonth | DateArg | PlainDateTime | ZonedDateTime): number;
    monthCode(arg: PlainYearMonth | PlainMonthDay | DateArg | PlainDateTime | ZonedDateTime): string;
    day(arg: PlainMonthDay | DateArg | PlainDateTime | ZonedDateTime): number;
    dayOfWeek(arg: DateArg | PlainDateTime | ZonedDateTime): number;
    dayOfYear(arg: DateArg | PlainDateTime | ZonedDateTime): number;
    weekOfYear(arg: DateArg | PlainDateTime | ZonedDateTime): number;
    daysInWeek(arg: DateArg | PlainDateTime | ZonedDateTime): number;
    daysInMonth(arg: PlainYearMonth | DateArg | PlainDateTime | ZonedDateTime): number;
    daysInYear(arg: PlainYearMonth | DateArg | PlainDateTime | ZonedDateTime): number;
    monthsInYear(arg: PlainYearMonth | DateArg | PlainDateTime | ZonedDateTime): number;
    inLeapYear(arg: PlainYearMonth | DateArg | PlainDateTime | ZonedDateTime): boolean;
    dateFromFields(arg: DateLikeFields, options?: OverflowOptions): PlainDate;
    yearMonthFromFields(arg: YearMonthLikeFields, options?: OverflowOptions): PlainYearMonth;
    monthDayFromFields(fields: MonthDayLikeFields, options?: OverflowOptions): PlainMonthDay;
    dateAdd(dateArg: DateArg, durationArg: DurationArg, options?: OverflowOptions): PlainDate;
    dateUntil(dateArg0: DateArg, dateArg1: DateArg, options?: {
        largestUnit?: DateUnit;
    }): Duration;
    toString(): string;
}

declare type CalendarArgSimple = CalendarProtocol | string;
declare type CalendarArgBag = {
    calendar: CalendarArgSimple;
};

interface CalendarDisplayMap {
    auto: 0;
    never: 1;
    always: 2;
}

interface DisambigMap {
    compatible: 0;
    earlier: 1;
    later: 2;
    reject: 3;
}

interface OffsetDisplayMap {
    auto: 0;
    never: 1;
}

interface OffsetHandlingMap {
    prefer: 0;
    use: 1;
    ignore: 2;
    reject: 3;
}

interface RoundingModeMap {
    halfExpand: RoundingFunc;
    ceil: RoundingFunc;
    trunc: RoundingFunc;
    floor: RoundingFunc;
}

declare type TimeZoneArgSimple = TimeZoneProtocol | string;
declare type TimeZoneArgBag = {
    timeZone: TimeZoneArgSimple;
};

interface TimeZoneDisplayMap {
    auto: 0;
    never: 1;
}

interface DurationTimeFields {
    hours: number;
    minutes: number;
    seconds: number;
    milliseconds: number;
    microseconds: number;
    nanoseconds: number;
}
interface DurationFields extends DurationTimeFields {
    years: number;
    months: number;
    weeks: number;
    days: number;
}

declare type YearMonthUnitProper = keyof YearMonthFields;
declare type DateUnitProper = keyof DateFields | 'week';
declare type TimeUnitProper = keyof TimeFields;

declare type CompareResult = -1 | 0 | 1;
declare type TimeUnit = TimeUnitProper | /** @deprecated */ 'hours' | /** @deprecated */ 'minutes' | /** @deprecated */ 'seconds' | /** @deprecated */ 'milliseconds' | /** @deprecated */ 'microseconds' | /** @deprecated */ 'nanoseconds';
declare type YearMonthUnit = YearMonthUnitProper | /** @deprecated */ 'years' | /** @deprecated */ 'months';
declare type DateUnit = DateUnitProper | /** @deprecated */ 'years' | /** @deprecated */ 'months' | /** @deprecated */ 'weeks' | /** @deprecated */ 'days';
declare type Unit = TimeUnit | DateUnit;
declare type DayTimeUnit = TimeUnit | 'day' | /** @deprecated */ 'days';
declare type RoundingMode = keyof RoundingModeMap;
declare type RoundingOptions<UnitType extends Unit = Unit> = {
    smallestUnit: UnitType;
    roundingMode?: RoundingMode;
    roundingIncrement?: number;
};
declare type TimeRoundingOptions = RoundingOptions<TimeUnit>;
declare type DateTimeRoundingOptions = RoundingOptions<DayTimeUnit>;
declare type DurationRoundingOptions = DiffOptions & {
    relativeTo?: ZonedDateTimeArg | DateTimeArg;
};
declare type DurationTotalOptions = {
    unit: Unit;
    relativeTo?: DateTimeArg;
};
declare type DiffOptions<UnitType extends Unit = Unit> = {
    largestUnit?: UnitType | 'auto';
    smallestUnit?: UnitType;
    roundingMode?: RoundingMode;
    roundingIncrement?: number;
};
declare type YearMonthDiffOptions = DiffOptions<YearMonthUnit>;
declare type DateDiffOptions = DiffOptions<DateUnit>;
declare type TimeDiffOptions = DiffOptions<TimeUnit>;
declare type CalendarDisplay = keyof CalendarDisplayMap;
declare type TimeZoneDisplay = keyof TimeZoneDisplayMap;
declare type OffsetDisplay = keyof OffsetDisplayMap;
declare type FractionalSecondDigits = 'auto' | 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9;
declare type DurationToStringUnit = 'second' | 'millisecond' | 'microsecond' | 'nanosecond' | /** @deprecated */ 'seconds' | /** @deprecated */ 'milliseconds' | /** @deprecated */ 'microseconds' | /** @deprecated */ 'nanoseconds';
declare type TimeToStringUnit = 'minute' | DurationToStringUnit | /** @deprecated */ 'minutes';
declare type DateToStringOptions = {
    calendarName?: CalendarDisplay;
};
declare type TimeToStringOptions<UnitType extends TimeUnit = TimeToStringUnit> = {
    fractionalSecondDigits?: FractionalSecondDigits;
    smallestUnit?: UnitType;
    roundingMode?: RoundingMode;
};
declare type DateTimeToStringOptions = DateToStringOptions & TimeToStringOptions;
declare type DurationToStringOptions = TimeToStringOptions<DurationToStringUnit>;
declare type InstantToStringOptions = TimeToStringOptions & {
    timeZone?: TimeZoneArg;
};
declare type ZonedDateTimeToStringOptions = DateTimeToStringOptions & {
    timeZoneName?: TimeZoneDisplay;
    offset?: OffsetDisplay;
};
declare type DateISOFields = DateISOEssentials & {
    calendar: CalendarProtocol;
};
declare type TimeISOFields = TimeISOEssentials & {
    calendar: CalendarProtocol;
};
declare type DateTimeISOFields = DateISOFields & TimeISOEssentials;
declare type ZonedDateTimeISOFields = DateTimeISOFields & {
    timeZone: TimeZone;
    offset: string;
};
declare type YearMonthLikeFields = ({
    era: string;
    eraYear: number;
} | {
    year: number;
}) & ({
    month: number;
} | {
    monthCode: string;
});
declare type MonthDayLikeFields = ({
    monthCode: string;
} | (({
    year: number;
} | {
    era: string;
    eraYear: number;
}) & {
    month: number;
})) & {
    day: number;
};
declare type DateLikeFields = YearMonthLikeFields & {
    day: number;
};
declare type DateTimeLikeFields = DateLikeFields & Partial<TimeFields>;
declare type ZonedDateTimeLikeFields = DateTimeLikeFields & {
    offset?: string;
};
declare type YearMonthLike = YearMonthLikeFields & {
    calendar?: CalendarArgSimple;
};
declare type MonthDayLike = {
    monthCode: string;
    calendar?: CalendarArgSimple;
} | {
    year: number;
    month: number;
    calendar?: CalendarArgSimple;
} | {
    era: string;
    eraYear: number;
    month: number;
    calendar: CalendarArgSimple;
} | {
    month: number;
    day: number;
    calendar?: never;
};
declare type DateLike = YearMonthLike & {
    day: number;
};
declare type TimeLike = Partial<TimeFields>;
declare type DateTimeLike = DateLike & TimeLike;
declare type ZonedDateTimeLike = DateTimeLike & {
    timeZone: TimeZoneArgSimple;
    offset?: string;
};
declare type DurationLike = Partial<DurationFields>;
declare type YearMonthArg = YearMonthLike | string;
declare type MonthDayArg = MonthDayLike | string;
declare type DateArg = DateLike | string;
declare type TimeArg = TimeLike | string;
declare type DateTimeArg = DateTimeLike | string;
declare type ZonedDateTimeArg = ZonedDateTimeLike | string;
declare type DurationArg = DurationLike | string;
declare type TimeZoneArg = TimeZoneArgSimple | TimeZoneArgBag;
declare type CalendarArg = CalendarArgSimple | CalendarArgBag;
declare type InstantArg = Instant | string | {
    toString(): string;
};
declare type LocalesArg = string | string[];
declare type YearMonthOverrides = Partial<YearMonthLikeFields>;
declare type MonthDayOverrides = Partial<MonthDayLikeFields>;
declare type DateOverrides = Partial<DateLikeFields>;
declare type DateTimeOverrides = Partial<DateTimeLikeFields>;
declare type ZonedDateTimeOverrides = Partial<ZonedDateTimeLikeFields>;
declare type OverflowHandling = keyof OverflowHandlingMap;
declare type OverflowOptions = {
    overflow?: OverflowHandling;
};
declare type Disambiguation = keyof DisambigMap;
declare type OffsetHandling = keyof OffsetHandlingMap;
declare type ZonedDateTimeOptions = {
    overflow?: OverflowHandling;
    disambiguation?: Disambiguation;
    offset?: OffsetHandling;
};
interface CalendarProtocol {
    id?: string;
    calendar?: never;
    era(arg: PlainYearMonth | DateArg | PlainDateTime | ZonedDateTime): string | undefined;
    eraYear(arg: PlainYearMonth | DateArg | PlainDateTime | ZonedDateTime): number | undefined;
    year(arg: PlainYearMonth | DateArg | PlainDateTime | ZonedDateTime): number;
    month(arg: PlainYearMonth | DateArg | PlainDateTime | ZonedDateTime): number;
    monthCode(arg: PlainYearMonth | PlainMonthDay | DateArg | PlainDateTime | ZonedDateTime): string;
    day(arg: PlainMonthDay | DateArg | PlainDateTime | ZonedDateTime): number;
    dayOfWeek?(arg: DateArg | PlainDateTime | ZonedDateTime): number;
    dayOfYear?(arg: DateArg | PlainDateTime | ZonedDateTime): number;
    weekOfYear?(arg: DateArg | PlainDateTime | ZonedDateTime): number;
    daysInWeek?(arg: DateArg | PlainDateTime | ZonedDateTime): number;
    daysInMonth?(arg: PlainYearMonth | DateArg | PlainDateTime | ZonedDateTime): number;
    daysInYear?(arg: PlainYearMonth | DateArg | PlainDateTime | ZonedDateTime): number;
    monthsInYear?(arg: PlainYearMonth | DateArg | PlainDateTime | ZonedDateTime): number;
    inLeapYear?(arg: PlainYearMonth | DateArg | PlainDateTime | ZonedDateTime): boolean;
    dateFromFields(arg: DateLikeFields, options?: OverflowOptions): PlainDate;
    yearMonthFromFields(arg: YearMonthLikeFields, options?: OverflowOptions): PlainYearMonth;
    monthDayFromFields(fields: MonthDayLikeFields, options?: OverflowOptions): PlainMonthDay;
    dateAdd?(dateArg: DateArg, durationArg: DurationArg, options?: OverflowOptions): PlainDate;
    dateUntil?(dateArg0: DateArg, dateArg1: DateArg, options?: {
        largestUnit?: DateUnit;
    }): Duration;
    toString(): string;
}
interface TimeZoneProtocol {
    id?: string;
    timeZone?: never;
    getOffsetNanosecondsFor(instantArg: InstantArg): number;
    getOffsetStringFor?(instantArg: InstantArg): string;
    getPlainDateTimeFor?(instantArg: InstantArg, calendarArg?: CalendarArg): PlainDateTime;
    getInstantFor?(dateTimeArg: DateTimeArg, options?: {
        disambiguation?: Disambiguation;
    }): Instant;
    getNextTransition?(instantArg: InstantArg): Instant | null;
    getPreviousTransition?(instantArg: InstantArg): Instant | null;
    getPossibleInstantsFor(dateTimeArg: DateTimeArg): Instant[];
    toString(): string;
    toJSON?(): string;
}

interface DateTemporalMethods {
    toTemporalInstant(): Instant;
}
declare type DateWithTemporal = Date & DateTemporalMethods;
declare function dateToTemporalInstant(date: Date): Instant;

interface DateTimeFormatRangePart extends Intl.DateTimeFormatPart {
    source: 'startDate' | 'endDate';
}
interface DateTimeFormatRangeMethods {
    formatRange(startDate: number | Date, endDate: number | Date): string;
    formatRangeToParts(startDate: number | Date, endDate: number | Date): DateTimeFormatRangePart[];
}
declare type DateTimeFormatWithRange = Intl.DateTimeFormat & DateTimeFormatRangeMethods;
declare type DateTimeFormatArg = number | Date | Instant | ZonedDateTime | PlainDateTime | PlainDate | PlainYearMonth | PlainMonthDay | PlainTime;
interface DateTimeFormatTemporalMethods {
    format(dateArg?: DateTimeFormatArg): string;
    formatToParts(dateArg?: DateTimeFormatArg): Intl.DateTimeFormatPart[];
    formatRange(startArg: DateTimeFormatArg, endArg: DateTimeFormatArg): string;
    formatRangeToParts(startArg: DateTimeFormatArg, endArg: DateTimeFormatArg): DateTimeFormatRangePart[];
}
declare type DateTimeFormatWithTemporal = Intl.DateTimeFormat & DateTimeFormatTemporalMethods;
declare const OrigDateTimeFormat: {
    (locales?: string | string[] | undefined, options?: Intl.DateTimeFormatOptions | undefined): Intl.DateTimeFormat;
    new (locales?: string | string[] | undefined, options?: Intl.DateTimeFormatOptions | undefined): Intl.DateTimeFormat;
    supportedLocalesOf(locales: string | string[], options?: Intl.DateTimeFormatOptions | undefined): string[];
};
declare function normalizeIntlOptionalDateArg(dateArg: DateTimeFormatArg | undefined): undefined | number | Date;
declare function normalizeIntlDateArg(dateArg: DateTimeFormatArg): number | Date;
declare function formatZoned(zdt: ZonedDateTime, locales: LocalesArg | undefined, options: Intl.DateTimeFormatOptions): string;
declare function formatUnzoned<ISOFields extends DateISOFields>(date: {
    getISOFields: () => ISOFields;
}, locales: LocalesArg | undefined, options: Intl.DateTimeFormatOptions, strictCalendar?: boolean): string;

declare function getZonedDateTimeISO(timeZoneArg?: TimeZoneArg): ZonedDateTime;
declare function getZonedDateTime(calendarArg: CalendarArg, timeZoneArg?: TimeZoneArg): ZonedDateTime;
declare function getPlainDateTimeISO(timeZoneArg?: TimeZoneArg): PlainDateTime;
declare function getPlainDateTime(calendarArg: CalendarArg, timeZoneArg?: TimeZoneArg): PlainDateTime;
declare function getPlainDateISO(timeZoneArg?: TimeZoneArg): PlainDate;
declare function getPlainDate(calendarArg: CalendarArg, timeZoneArg?: TimeZoneArg): PlainDate;
declare function getPlainTimeISO(timeZoneArg?: TimeZoneArg): PlainTime;
declare function getInstant(): Instant;
declare function getTimeZone(): TimeZone;
declare const Now: {
    zonedDateTimeISO: typeof getZonedDateTimeISO;
    zonedDateTime: typeof getZonedDateTime;
    plainDateTimeISO: typeof getPlainDateTimeISO;
    plainDateTime: typeof getPlainDateTime;
    plainDateISO: typeof getPlainDateISO;
    plainDate: typeof getPlainDate;
    plainTimeISO: typeof getPlainTimeISO;
    instant: typeof getInstant;
    timeZone: typeof getTimeZone;
};

export { Calendar, CalendarArg, CalendarDisplay, CalendarProtocol, CompareResult, DateArg, DateDiffOptions, DateISOFields, DateLike, DateLikeFields, DateOverrides, DateTemporalMethods, DateTimeArg, DateTimeFormatArg, DateTimeFormatRangeMethods, DateTimeFormatRangePart, DateTimeFormatTemporalMethods, DateTimeFormatWithRange, DateTimeFormatWithTemporal, DateTimeISOFields, DateTimeLike, DateTimeLikeFields, DateTimeOverrides, DateTimeRoundingOptions, DateTimeToStringOptions, DateToStringOptions, DateUnit, DateWithTemporal, DayTimeUnit, DiffOptions, Disambiguation, Duration, DurationArg, DurationLike, DurationRoundingOptions, DurationToStringOptions, DurationToStringUnit, DurationTotalOptions, FractionalSecondDigits, Instant, InstantArg, InstantToStringOptions, LocalesArg, MonthDayArg, MonthDayLike, MonthDayLikeFields, MonthDayOverrides, Now, OffsetDisplay, OffsetHandling, OrigDateTimeFormat, OverflowHandling, OverflowOptions, PlainDate, PlainDateTime, PlainMonthDay, PlainTime, PlainYearMonth, RoundingMode, RoundingOptions, TimeArg, TimeDiffOptions, TimeISOFields, TimeLike, TimeRoundingOptions, TimeToStringOptions, TimeToStringUnit, TimeUnit, TimeZone, TimeZoneArg, TimeZoneDisplay, TimeZoneProtocol, Unit, YearMonthArg, YearMonthDiffOptions, YearMonthLike, YearMonthLikeFields, YearMonthOverrides, YearMonthUnit, ZonedDateTime, ZonedDateTimeArg, ZonedDateTimeISOFields, ZonedDateTimeLike, ZonedDateTimeLikeFields, ZonedDateTimeOptions, ZonedDateTimeOverrides, ZonedDateTimeToStringOptions, dateToTemporalInstant, formatUnzoned, formatZoned, normalizeIntlDateArg, normalizeIntlOptionalDateArg };
